@@ -6,36 +6,12 @@
 #define random lbl_random
 #define srandom lbl_srandom
 
-#include <winsock.h>
 #include "search.h"
 
 #define close closesocket
-
 #define MAXHOSTNAMELEN  256
-
 #define _SYS_NMLN       9
 
-
-struct utsname {
-        char sysname[_SYS_NMLN];
-        char nodename[_SYS_NMLN];
-        char release[_SYS_NMLN];
-        char version[_SYS_NMLN];
-        char machine[_SYS_NMLN];
-};
-                                       
-typedef char *caddr_t;
-
-struct iovec {
-        caddr_t iov_base;
-        int         iov_len;
-};
-
-struct timezone {
-        int tz_minuteswest;
-        int tz_dsttime;
-};
-
 typedef int pid_t;
 typedef int uid_t;
 typedef int gid_t;
