@@ -2,27 +2,37 @@
 #set src_dir /cs/research/mice/speedy/home/mhandley/tcl
 #source $src_dir/net_balloon.tcl
 
-#Use same colours as vat
+#Use same colours as rat
 #general
-option add *background gray80 widgetDefault
-option add *foreground black widgetDefault
-option add *activeBackground gray95 widgetDefault
-option add *selectBackground gray95 widgetDefault
-option add *scrollbarBackground gray50 widgetDefault
-option add *scrollbarForeground gray80 widgetDefault
-option add *scrollbarActiveForeground gray95 widgetDefault
-option add *canvasBackground gray95 widgetDefault
 #option add *tixBalloonBg gray50
 #option add *tixBalloonFg white
 #test
+
+option add *foreground black startupFile
+option add *activeBackground gray85 startupFile
+option add *selectBackground gray85 startupFile
+option add *scrollbarBackground gray50 startupFile
+option add *scrollbarForeground gray80 startupFile
+option add *scrollbarActiveForeground gray95 startupFile
+option add *Checkbutton.anchor w startupFile
+option add *Radiobutton.anchor w startupFile
+option add *Radiobutton.relief flat startupFile
+option add *Scale.sliderForeground gray80 startupFile
+option add *Scale.activeForeground gray85 startupFile
+option add *Scale.background gray80 startupFile
+option add *selector forestgreen startupFile
 option add *disabledBackground gray80 widgetDefault
+option add *canvasBackground gray95 widgetDefault
 option add *disabledForeground gray50 widgetDefault
 option add *hotForeground blue widgetDefault
 option add *activehotForeground red widgetDefault
 option add *balloonHelp 0
 option add *fontSize normal widgetDefault
 option add *participants 1 widgetDefault
+
 if {$tcl_platform(platform) == "unix"} {
+	. configure -background gray80
+	option add *background gray80 startupFile
 	option add *font \
 	 -*-helvetica-bold-r-normal--12-*-$charset \
 	 widgetDefault
@@ -36,6 +46,8 @@ if {$tcl_platform(platform) == "unix"} {
 	  -*-helvetica-bold-r-normal--10-*-$charset \
 	 widgetDefault
 } else {
+	. configure -background gray75
+	option add *background gray75 startupFile
 	option add *font \
 	  -*-helvetica-bold-r-normal--12-*-iso8859-1 \
 	  widgetDefault
@@ -70,6 +82,109 @@ frame .m.l.txt -borderwidth 2 -relief sunken
 
 label .m.cp.l -text "UCL Network Text Editor $NT_VERSION"
 
+############################################
+# setup fonts
+############################################
+proc init_fonts {size newfh newfw} {
+	global fhu fwu fh fw charset tcl_platform font cur_font
+
+	if {$tcl_platform(platform) == "unix"} {
+		set fonts [list -adobe-courier-medium-r-normal--*-$size-75-75-*-*-$charset\
+			-adobe-courier-medium-o-normal--*-$size-75-75-*-*-$charset\
+			-adobe-courier-bold-r-normal--*-$size-75-75-*-*-$charset\
+			-adobe-courier-bold-o-normal--*-$size-75-75-*-*-$charset]
+		set fontnames {Normal Italic Bold "Bold Italic"}
+	} else {
+		set fonts [font families]
+		set result ""
+		foreach f $fonts {
+			if {[string match "*Courier New" $f]} {
+				lappend result $f
+				}
+		}
+		set result [lsort $result]
+		set fonts ""
+		foreach f $result {
+			lappend fonts "{$f} {$size} normal"
+			lappend fonts "{$f} {$size} normal italic"
+			lappend fonts "{$f} {$size} bold"
+			lappend fonts "{$f} {$size} bold italic"
+		}		
+
+		set fontnames ""
+		foreach f $result {
+			lappend fontnames "$f Normal"
+			lappend fontnames "$f Italic" 
+			lappend fontnames "$f Bold" 
+			lappend fontnames "$f Bold Italic"
+		}
+	}
+
+	set i 0
+	foreach fontn $fontnames {
+	 set fontname($i) $fontn
+	 incr i
+	}
+	set i 0
+
+	set fontsize [option get . fontSize nt]
+
+	#these should really be derived automatically
+	set fw $newfw
+	set fh $newfh
+
+	# font height and width constants that shouldn't be changed.
+	set fwu 7
+	set fhu 13
+
+	# delete the menu if it exists
+	.m.cp.font.m delete 0 999
+	foreach fonti $fonts {
+		set font($i) $fonti
+		.m.cp.font.m add command -label $fontname($i) -font $fonti \
+			-command "set_font $i"
+		incr i
+	}
+	.m.cp.font.m entryconfigure $cur_font -background gray85
+}
+
+proc view {size newfh newfw} {
+	global fh fw charset tcl_platform
+
+	# for each of the text items on the canvas change the size and re-position them
+	foreach item [.m.l.txt.c find all] {
+		catch {
+			set font [.m.l.txt.c itemconfigure $item -font]
+			if {$tcl_platform(platform) == "unix"} {
+				if {[string match *medium-r* $font]} {.m.l.txt.c itemconfigure $item -font "-adobe-courier-medium-r-normal--*-$size-75-75-*-*-$charset"}
+				if {[string match *medium-o* $font]} {.m.l.txt.c itemconfigure $item -font "-adobe-courier-medium-o-normal--*-$size-75-75-*-*-$charset"}
+				if {[string match *bold-r* $font]} {.m.l.txt.c itemconfigure $item -font "-adobe-courier-bold-r-normal--*-$size-75-75-*-*-$charset"}
+				if {[string match *bold-o* $font]} {.m.l.txt.c itemconfigure $item -font "-adobe-courier-bold-o-normal--*-$size-75-75-*-*-$charset"}
+			} else {
+				if {[string match *normal* $font]} {.m.l.txt.c itemconfigure $item -font "{[lindex [lindex $font 4] 0]} $size normal"}
+				if {[string match *bold* $font]} {.m.l.txt.c itemconfigure $item -font "{[lindex [lindex $font 4] 0]} $size bold"}
+				if {[string match *italic* $font]} {.m.l.txt.c itemconfigure $item -font "{[lindex [lindex $font 4] 0]} $size italic"}
+				if {[string match {*bold italic*} $font]} {.m.l.txt.c itemconfigure $item -font "{[lindex [lindex $font 4] 0]} $size bold italic"}
+			}
+			set x [lindex [.m.l.txt.c coords $item] 0]
+			set y [lindex [.m.l.txt.c coords $item] 1]
+			set newy [expr $y/$fh]
+			set newy [expr $newy*$newfh]	
+			set newx [expr $x/$fw]
+			set newx [expr $newx*$newfw]
+			.m.l.txt.c coords $item $newx $newy	
+		}
+	}
+	init_fonts $size $newfh $newfw
+	grow_canvas
+	update
+}
+
+
+############################################
+# menus
+############################################
+
 menubutton .m.cp.file -text "File" -width 6 -menu .m.cp.file.m -relief raised \
     -borderwidth 1
 menu .m.cp.file.m -tearoff 0
@@ -77,13 +192,7 @@ menu .m.cp.file.m -tearoff 0
 .m.cp.file.m add command -label "Load Structured Text" -command "load_text struct"
 .m.cp.file.m add command -label "Save Plain Text" -command "save_text plain"
 .m.cp.file.m add command -label "Save Structured Text" -command "save_text struct"
-set check_sum_active 1
-.m.cp.file.m add checkbutton -label "Checksum Protocols" -variable check_sum_active \
-    -command {set_checksum_active $check_sum_active}
 .m.cp.file.m add command -label "Quit                   (Ctrl-Q)" -command "quit"
-
-
-
 wm protocol . WM_DELETE_WINDOW exit
 
 menubutton .m.cp.edit -text "Edit" -width 6 -relief raised -borderwidth 1 \
@@ -122,96 +231,63 @@ menubutton .m.cp.font -text Font -width 6 -menu .m.cp.font.m -relief raised \
     -borderwidth 1
 menu .m.cp.font.m -tearoff 0
 
-if {$tcl_platform(platform) == "unix"} {
-   set fonts [list -adobe-courier-medium-r-normal--*-120-75-75-*-*-$charset\
-                -adobe-courier-medium-o-normal--*-120-75-75-*-*-$charset\
-                -adobe-courier-bold-r-normal--*-120-75-75-*-*-$charset\
-                -adobe-courier-bold-o-normal--*-120-75-75-*-*-$charset]
-   set mediumfonts [list -adobe-courier-medium-r-normal--14-*-$charset \
-                      -adobe-courier-medium-o-normal--14-*-$charset \
-                      -adobe-courier-bold-r-normal--14-*-$charset \
-                      -adobe-courier-bold-o-normal--14-*-$charset ]
-   set largefonts [list -adobe-courier-medium-r-normal--24-*-$charset \
-                     -adobe-courier-medium-o-normal--24-*-$charset \
-                     -adobe-courier-bold-r-normal--24-*-$charset \
-                     -adobe-courier-bold-o-normal--24-*-$charset ]
-} else {
-# Make sure that we get the correct font.  Using a XLFD doesn't
-# guarantee that we will get Courier New (TT).
-set fonts {\
-"{Courier New} 9 normal" \
-"{Courier New} 9 normal italic" \
-"{Courier New} 9 bold" \
-"{Courier New} 9 bold italic" }
-}
-
-set fontnames {\
-Normal \
-Italic \
-Bold \
-"Bold Italic"}
-set i 0
-foreach fontn $fontnames {
-    set fontname($i) $fontn
-    incr i
-}
-set i 0
+# set current font to bold
 set cur_font 2
-set fontsize [option get . fontSize nt]
-#these should really be derived automatically
 
 if {$tcl_platform(platform) == "unix"} {
-   if {$fontsize=="normal"} {
-      set fw 7
-      set fh 13
-      set currentfonts $fonts
-   } elseif {$fontsize=="medium"} {
-      set fw 9
-      set fh 15
-      set currentfonts $mediumfonts
-   } else {
-      set fw 7
-      set fh 13
-      set currentfonts $fonts
-   }
+	init_fonts 120 13 7
 } else {
-# Font height is different in Windows - I have proved that this is the fh win32 uses
-      set fw 7
-      set fh 16
-	  set currentfonts $fonts
+	init_fonts 9 16 7
 }
 
-# fh is *now* used to position text on the canvas
-# fhu is used to calculate pointer (ie hand, eraser, pen) position
-# Note: This should always be 13 so that the pointer is correctly mapped
-# on machines that use a font height of 13 (unix)
-set fhu 13
-
-foreach fonti $currentfonts {
-    set font($i) $fonti
-    .m.cp.font.m add command -label $fontname($i) -font $fonti \
-	-command "set_font $i"
-    incr i
-}
-
-menubutton .m.cp.show -text Show -width 6 -menu .m.cp.show.m  -relief raised \
+menubutton .m.cp.view -text View -width 6 -menu .m.cp.view.m  -relief raised \
     -borderwidth 1
-menu .m.cp.show.m -tearoff 0
+menu .m.cp.view.m -tearoff 0
+
+# view proc: point size, font height, font width
+set view 1
+if {$tcl_platform(platform) == "unix"} {
+	.m.cp.view.m add radiobutton -label "100%" -command {view 120 13 7} -variable view -value 1
+	.m.cp.view.m add radiobutton -label "111%" -command {view 130 14 8} -variable view
+	.m.cp.view.m add radiobutton -label "122%" -command {view 140 15 9} -variable view
+	.m.cp.view.m add radiobutton -label "133%" -command {view 150 16 9} -variable view
+	.m.cp.view.m add radiobutton -label "144%" -command {view 160 17 10} -variable view
+	.m.cp.view.m add radiobutton -label "155%" -command {view 170 18 11} -variable view
+} else {
+	.m.cp.view.m add radiobutton -label "100%" -command {view 9 16 7} -variable view -value 1
+	.m.cp.view.m add radiobutton -label "111%" -command {view 10 16 8} -variable view 
+	.m.cp.view.m add radiobutton -label "122%" -command {view 11 17 9} -variable view 
+	.m.cp.view.m add radiobutton -label "133%" -command {view 12 18 10} -variable view 
+	.m.cp.view.m add radiobutton -label "144%" -command {view 13 20 10} -variable view 
+	.m.cp.view.m add radiobutton -label "155%" -command {view 14 22 11} -variable view 
+}
+
+menubutton .m.cp.options -text Options -width 6 -menu .m.cp.options.m  -relief raised \
+    -borderwidth 1
+menu .m.cp.options.m -tearoff 0
 set part_vis [option get . participants nt]
-.m.cp.show.m add checkbutton -label "Participants" -variable part_vis \
+.m.cp.options.m add checkbutton -label "Participants List" -variable part_vis \
     -command {show_participants $part_vis}
 set block_vis 1
-.m.cp.show.m add checkbutton -label "Current Block Info" -variable block_vis \
+.m.cp.options.m add checkbutton -label "Current Block Info" -variable block_vis \
     -command {show_block_info $block_vis}
+set check_sum_active 0
+.m.cp.options.m add checkbutton -label "Checksum Protocols" -variable check_sum_active \
+    -command {set_checksum_active $check_sum_active} -state disabled
+set deselect_blocks 1
+.m.cp.options.m add checkbutton -label "Deselect blocks when idle" -variable deselect_blocks \
+	-command {check_activity}
+set minimise_updates 1
+.m.cp.options.m add checkbutton -label "Minimise screen updates" -variable minimise_updates \
+	-command {ui_active_block $minimise_updates}
 
 button .m.cp.help -text Help -width 6 -relief raised \
-    -borderwidth 1 -command {webdisp help:about} -padx 3 -pady 3
+    -borderwidth 1 -command {webdisp help:about}
 
-pack .m.cp.file .m.cp.edit .m.cp.col .m.cp.font .m.cp.show -side left
+pack .m.cp.file .m.cp.edit .m.cp.col .m.cp.font .m.cp.view .m.cp.options -side left -fill y
 pack .m.cp.help -side right -padx 0 -pady 0
 pack .m.cp.l -side right -expand true -fill x
 
-
 label .m.in.l -text "Block:" -font $infofont
 label .m.in.bid -text "None" -font $infofont -width 16 -relief sunken \
     -borderwidth 1
@@ -269,7 +345,7 @@ checkbutton .m.l.ss.readonly -text "Read Only" -relief flat \
     -anchor w -highlightthickness 0
 checkbutton .m.l.ss.locking -text "Lock New Items" -relief flat \
     -font $userfont -borderwidth 1 -command {set_lock_mode $locking} \
-    -anchor w -highlightthickness 0
+    -anchor w -highlightthickness 0 -state disabled
 checkbutton .m.l.ss.encrypt -text "Encrypt Session" -relief flat \
     -font $userfont -borderwidth 1 -command {set_key $encrypt} \
     -anchor w -highlightthickness 0
@@ -297,9 +373,11 @@ proc hide_participants {} {
 }
 proc show_participants {flag} {
     if {$flag==1} {
-	pack .m.l.ss -before .m.l.txt -side right -fill both
+		pack forget .m.l.hsb
+		pack .m.l.ss -before .m.l.txt -side right -fill both
+		pack .m.l.hsb -side bottom -fill x -padx 2
     } else {
-	pack forget .m.l.ss
+		pack forget .m.l.ss
     }
 }
 ############################################
@@ -307,7 +385,9 @@ proc show_participants {flag} {
 ############################################
 
 set canvypos 0
+set canvxpos 0
 
+# smaller canvas size for windows
 if {$tcl_platform(platform)=="windows"} {
 	canvas .m.l.txt.c -height [expr 32 * $fh] -width [expr 90 * $fw] \
 		-relief flat \
@@ -320,15 +400,30 @@ if {$tcl_platform(platform)=="windows"} {
 		-confine true
 }
 
-scrollbar .m.l.txt.sb -command "scroll_canvas" \
+scrollbar .m.l.txt.sb -command "scroll_canvas_y" \
     -background [option get . background Mtool] \
     -troughcolor [option get . scrollbarBackground Mtool] \
     -borderwidth 1 
+
+scrollbar .m.l.hsb -command "scroll_canvas_x" \
+    -background [option get . background Mtool] \
+    -troughcolor [option get . scrollbarBackground Mtool] \
+    -borderwidth 1 -orient horizontal
+
 wnamemap .m.l.txt.c main-editor-panel
-wnamemap .m.l.txt.sb main-editor-panel-scrollbar
-.m.l.txt.c addtag margin withtag \
-    [.m.l.txt.c create line [expr 90 * $fw] 0 [expr 90 * $fw] 520\
-    -stipple dashed -width 3]
+wnamemap .m.l.txt.sb main-editor-panel-scrollbar_y
+wnamemap .m.l.hsb main-editor-panel-scrollbar_x
+
+# The tcl/tk canvas speedup for windows doesn't support dashed lines
+if {$tcl_platform(platform) == "unix"} {
+	.m.l.txt.c addtag margin withtag \
+	  [.m.l.txt.c create line [expr 90 * $fw] 0 [expr 90 * $fw] 520\
+	 -stipple dashed -width 3]
+} else {
+	.m.l.txt.c addtag margin withtag \
+	  [.m.l.txt.c create line [expr 90 * $fw] 0 [expr 90 * $fw] 520\
+	 -fill gray80 -width 2]
+}
 
 #activity indicator
 set activity_indicator ".m.l.ss.c2"
@@ -378,40 +473,41 @@ proc int {i} {
     return [lindex [split $i "."] 0]
 }
 proc grow_canvas {} {
-    global fw fh canvypos
+    global fw fh canvypos canvxpos
     set ymax [lindex [.m.l.txt.c bbox all] 3]
     if {$ymax == ""} { set ymax 0.0 }
     set top [.m.l.txt.c canvasy 0]
+	set left [.m.l.txt.c canvasx 0]
+	set canvxpos [int $left]
     set canvypos [int $top]
     set height [lindex [.m.l.txt.c configure -height] 4]
     set bottom [.m.l.txt.c canvasy $height]
     if { ($ymax < $bottom) && ($ymax<$height) } {
-	set ymax $bottom
+		set ymax $bottom
     }
-    .m.l.txt.c configure -scrollregion "0.0 0.0 520.0 $ymax"
+    .m.l.txt.c configure -scrollregion "0.0 0.0 {[expr $fw*110]} $ymax"
     if {$top < -$fh} {
-	.m.l.txt.c yview scroll [expr -[int $top]/$fh] units
-#	puts "at top"
+		.m.l.txt.c yview scroll [expr -[int $top]/$fh] units
     }
     if {$ymax <= $bottom} {
-	.m.l.txt.c yview moveto [expr 1.0-($height.0/$ymax)]
-#	puts "at bottom"
+		.m.l.txt.c yview moveto [expr 1.0-($height.0/$ymax)]
     }
     .m.l.txt.sb set [lindex [.m.l.txt.c yview] 0] [lindex [.m.l.txt.c yview] 1]
+	.m.l.hsb set [lindex [.m.l.txt.c xview] 0] [lindex [.m.l.txt.c xview] 1]
     set_activity $ymax $height $top $bottom
     .m.l.txt.c coords margin [expr 90 * $fw] 0 [expr 90 * $fw] [expr $ymax -4]
 }
 
-#call it now to initialise the scrollbar...
-#update
-#grow_canvas
-
-
-proc scroll_canvas args {
+proc scroll_canvas_y args {
     eval ".m.l.txt.c yview $args"
     grow_canvas
 }
 
+proc scroll_canvas_x args {
+    eval ".m.l.txt.c xview $args"
+    grow_canvas
+}
+
 #  -font -Adobe-Courier-Bold-R-Normal-*-120-* \
 #  -yscroll ".m.l.txt.sb set" -wrap none
 #scrollbar .m.l.txt.sb -command ".m.l.txt.t yview" \
@@ -419,9 +515,32 @@ proc scroll_canvas args {
 #  -foreground [option get . scrollbarForeground Mtool] \
 #  -activeforeground [option get . scrollbarActiveForeground Mtool]
 
+# check if there's been any activity in the last 60 secs
+# if not then deactivate any blocks to reduce canvas updates.
+
+proc check_activity {} {
+	global last_update deselect_blocks
+	if {$deselect_blocks == 0} {return}
+	set curtime [expr [clock seconds] - 60]
+	if {$curtime <=60} {after 60000 {check_activity}}
+	if {$curtime >= $last_update} {
+		.m.in.cid configure -text ""
+		.m.in.mid configure -text ""
+		.m.in.mt configure -text ""
+		.m.in.bid configure -text "none" -foreground black
+		focus .
+		ui_active_block NULL
+		after 60000 {check_activity}
+	} else {
+		after [expr $last_update - $curtime + 1] {check_activity}
+	}
+}
+set last_update [clock seconds]
+check_activity
+
 proc set_info_display {} {
     global cur_block exists
-	
+
 	if {$cur_block=="bid0"}	{
 	return
 	}
@@ -446,7 +565,7 @@ proc set_info_display {} {
 		#if cur_block is deleted
 		.m.in.mid configure -text ""
 		.m.in.mt configure -text ""
-		.m.in.bid configure -text "none"
+		.m.in.bid configure -text "none" 
 	} else {
 		#if cur_block not deleted
 		.m.in.mid configure -text [ui_get_modifier $cur_block]
@@ -521,21 +640,21 @@ bind .m.l.txt.c <Control-x> {cut_block $cur_block}
 bind .m.l.txt.c <Control-c> {copy_block $cur_block}
 bind .m.l.txt.c <Control-v> {paste_block}
 
-bind .m.l.txt.c <1> {findblock %x [expr %y + $canvypos]}
+bind .m.l.txt.c <1> {findblock [expr %x + $canvxpos] [expr %y + $canvypos]}
 if {$tcl_platform(platform)=="unix"} {
-    bind .m.l.txt.c  <B2-Motion> {moveblock %x [expr %y + $canvypos]}
+    bind .m.l.txt.c  <B2-Motion> {moveblock [expr %x + $canvxpos] [expr %y + $canvypos]}
     bind .m.l.txt.c <ButtonRelease-2> {grow_canvas;clear_move_block}
 } else {
 
 # A hack to get the control key and mouse button to move text in Windows
     bind .m.l.txt.c <KeyPress-Control_L> {bind .m.l.txt.c <1> ""}
     bind .m.l.txt.c <KeyRelease-Control_L> {reset_binding}
-    bind .m.l.txt.c  <Control-B1-Motion> {moveblock %x [expr %y + $canvypos]}
+    bind .m.l.txt.c  <Control-B1-Motion> {moveblock [expr %x + $canvxpos] [expr %y + $canvypos]}
     bind .m.l.txt.c <Control-ButtonRelease-1> {grow_canvas;clear_move_block}
 }
 
 proc reset_binding {} {
-	bind .m.l.txt.c <1> {findblock %x [expr %y + $canvypos]}
+	bind .m.l.txt.c <1> {findblock [expr %x + $canvxpos] [expr %y + $canvypos]}
 }
 
 #the following binding would display all of the block id's in the page, but would then
@@ -545,8 +664,8 @@ proc reset_binding {} {
 #replace with this 'useless' binding to prevent ctrl-i doing funny things.
 bind .m.l.txt.c <Control-i> ""
 
-bind .m.l.txt.c <3> {create_my_shared_pointer %x [expr %y + $canvypos]}
-bind .m.l.txt.c <B3-Motion> {move_my_shared_pointer %x [expr %y + $canvypos]}
+bind .m.l.txt.c <3> {create_my_shared_pointer [expr %x + $canvxpos] [expr %y + $canvypos]}
+bind .m.l.txt.c <B3-Motion> {move_my_shared_pointer [expr %x + $canvxpos] [expr %y + $canvypos]}
 bind .m.l.txt.c <ButtonRelease-3> {delete_my_shared_pointer}
 
 proc set_readable_mode {flag} {
@@ -572,8 +691,8 @@ proc read_write {} {
     bind .m.l.txt.c <Any-KeyPress>  "insert_char %x %y %A "
     bind .m.l.txt.c <Return> insert_cr
     bind .m.l.txt.c <Delete> delete_char
-    bind .m.l.txt.c  <B2-Motion> {moveblock %x [expr %y + $canvypos]}
-    bind .m.l.txt.c <1> {findblock %x [expr %y + $canvypos]}
+    bind .m.l.txt.c  <B2-Motion> {moveblock [expr %x + $canvxpos] [expr %y + $canvypos]}
+    bind .m.l.txt.c <1> {findblock [expr %x + $canvxpos] [expr %y + $canvypos]}
     pack .m.cp.edit -side left -after .m.cp.file
     pack .m.cp.col -side left -after .m.cp.edit
     pack .m.cp.font -side left -after .m.cp.col
@@ -610,11 +729,12 @@ proc set_col {coli} {
 }
 
 proc set_font {fonti} {
-    global cur_block
-    global font
-    global cur_font exists
+    global cur_block font exists cur_font
+	
+	.m.cp.font.m entryconfigure $cur_font -background ""
+	.m.cp.font.m entryconfigure $fonti -background gray85
 
-    set cur_font $fonti
+	set cur_font $fonti
 
     set tmp $exists($cur_block)
 	if {$tmp==0} {
@@ -660,7 +780,7 @@ proc delete_block {id} {
 
 #called from menu to actually delete it
 proc try_to_delete_block {id} {
-    global xpos ypos fw fh cur_col fhu
+    global xpos ypos fw fh cur_col fhu fwu
 	global exists
 
 	
@@ -674,15 +794,15 @@ proc try_to_delete_block {id} {
 	display_lock $id
 	return
     } else {
-	ui_send_shared_pointer [expr $xpos($id) * $fw] \
+	ui_send_shared_pointer [expr $xpos($id) * $fwu] \
 	    [expr $ypos($id) * $fhu] $cur_col eraser
-	after 250 ui_send_shared_pointer [expr $xpos($id) * $fw + 20] \
+	after 250 ui_send_shared_pointer [expr $xpos($id) * $fwu + 20] \
 	    [expr $ypos($id) * $fhu]  $cur_col eraser
-	after 500 ui_send_shared_pointer [expr $xpos($id) * $fw + 40] \
+	after 500 ui_send_shared_pointer [expr $xpos($id) * $fwu + 40] \
 	    [expr $ypos($id) * $fhu] $cur_col eraser
-	after 750 ui_send_shared_pointer [expr $xpos($id) * $fw + 20] \
+	after 750 ui_send_shared_pointer [expr $xpos($id) * $fwu + 20] \
 	    [expr $ypos($id) * $fhu] $cur_col eraser
-	after 1000 ui_send_shared_pointer [expr $xpos($id) * $fw] \
+	after 1000 ui_send_shared_pointer [expr $xpos($id) * $fwu] \
 	    [expr $ypos($id) * $fhu] $cur_col eraser
 	ui_delete_block $id
 	delete_block $id
@@ -695,9 +815,20 @@ proc try_to_delete_block {id} {
 
 	set_info_display
 	grow_canvas
-    }
+    }	
+}
 
+proc set_delete_status {id} {
+	global exists
+
+	set tmp $exists($id)
+	if {$tmp==0} {return}
 	
+	delete_block $id
+	# set status of this block as deleted, remove our knowledge of it..
+	# I am using exists as 1=block has been created and still exists
+	# 0=block has been created and has now been deleted.
+	set exists($id) 0	
 }
 
 #called from menu to try to undo last block deletion
@@ -770,10 +901,10 @@ proc paste_block {} {
 }
 
 proc startmoveblock {block x y} {
-    global cur_block fh fw
-    global cursor_x cursor_y send_y
+    global cur_block
+    global cursor_x cursor_y send_y send_x
     global xpos ypos
-    global fw fh fhu
+    global fw fh
     global px py
     global move_block_end
     global old_block
@@ -790,6 +921,7 @@ proc startmoveblock {block x y} {
 	#jim change
 	#set send_y [expr $y - ($ypos($block) * $fhu)]
 	set send_y [expr $y - ($ypos($block) * $fh)]
+	set send_x [expr $x - ($xpos($block) * $fw)]
 
     set px 0
     set py 0
@@ -802,8 +934,8 @@ proc startmoveblock {block x y} {
 }
 
 proc moveblock {x y} {
-    global cur_block fh fw fhu
-    global cursor_x cursor_y xpos ypos send_y
+    global cur_block fh fw fhu fwu
+    global cursor_x cursor_y xpos ypos send_y send_x
     global cur_col
     global px py
     global move_block_end
@@ -822,10 +954,12 @@ proc moveblock {x y} {
 
 # adjust pointer...
     set send_ny [expr $y - $send_y]
+	set send_nx [expr $x - $send_x]
 	#jim change
     #set send_ny [expr $send_ny -($send_ny%$fh)]
     
 	if {($send_ny < 0)} { set send_ny 0}
+	if {($send_nx < 0)} { set send_nx 0}
 
     if {($nx!=$px)|($ny!=$py)} {
 	
@@ -833,7 +967,10 @@ proc moveblock {x y} {
 	set newy [expr $send_ny/$fh]
 	set newy [expr $newy*$fhu]	
 	
-	ui_send_shared_pointer $nx $newy $cur_col hand
+	set newx [expr $send_nx/$fw]
+	set newx [expr $newx*$fwu]	
+
+	ui_send_shared_pointer $newx $newy $cur_col hand
 	#ui_send_shared_pointer $nx $send_ny $cur_col hand
 	#end of jim's changes 
 	set_posn $nx $ny
@@ -856,7 +993,7 @@ proc clear_move_block {} {
 proc delete_char2 {} {
   global cur_block exists
   global xpos ypos
-  global xp yp fhu
+  global xp yp fhu last_update
   global curline curlnum fw fh cur_col
   
 	set tmp $exists($cur_block)
@@ -867,6 +1004,8 @@ proc delete_char2 {} {
       display_lock $cur_block
       return 
   }
+
+  set last_update [clock seconds]
   set pos [.m.l.txt.c index $cur_block insert]
   set txt [lindex [.m.l.txt.c itemconfigure $cur_block -text] 4]
   set xcpos [.m.l.txt.c index $cur_block xinsert]
@@ -1058,17 +1197,17 @@ proc move_down {} {
 proc insert_cr {} {
   global cur_block exists 
   global curline curlnum
-  global max_block_size;
+  global max_block_size last_update
 
-  	set tmp $exists($cur_block)
-	if {$tmp==0} {
-		return }
+  set tmp $exists($cur_block)
+  if {$tmp==0} {return}
   set lock [ui_get_my_lock_status $cur_block]
   if { $lock == "LOCKED" } { 
       display_lock $cur_block
       return 
   }
-
+  
+  set last_update [clock seconds]
   #default max_block_size is 200, but allow us to change it if user changes their mind
   set max_block_size [ui_get_max_block_size]
 
@@ -1141,7 +1280,7 @@ proc insert_char {x y A} {
   global xpos ypos
   global xp yp
   global curline curlnum cur_col
-  global fh fw fhu
+  global fh fw fhu fwu
   global exists
   set lock [ui_get_my_lock_status $cur_block]
 
@@ -1169,6 +1308,7 @@ proc insert_char {x y A} {
       insert_cr
   }
 
+  if {[string compare $A "\n"] == 0} {return}
   .m.l.txt.c insert $cur_block insert $A
   set pos [.m.l.txt.c index $cur_block insert]
   set xcpos [.m.l.txt.c index $cur_block xinsert]
@@ -1178,12 +1318,10 @@ proc insert_char {x y A} {
   set lend [.m.l.txt.c index $cur_block lineend]
   set curline [string range $txt $lstart $lend]
   catch {ui_set_line $cur_block $ycpos $curline}
-  ui_send_shared_pointer [expr $fw * ($xpos($cur_block) + $xcpos)] \
+  ui_send_shared_pointer [expr $fwu * ($xpos($cur_block) + $xcpos)] \
       [expr $fhu * ($ypos($cur_block) + $ycpos)]\
 	  $cur_col pen
   set_info_display
-
-
 }
 
 proc get_line {txt pos} {
@@ -1209,7 +1347,7 @@ proc get_line {txt pos} {
 proc delete_char {} {
   global cur_block exists
   global xpos ypos
-  global xp yp fhu
+  global xp yp fhu fwu
   global curline curlnum fw fh cur_col
   
 	set tmp $exists($cur_block)
@@ -1242,7 +1380,7 @@ proc delete_char {} {
   set lend [.m.l.txt.c index $cur_block lineend]
   set curline [string range $txt $lstart $lend]
   ui_set_line $cur_block $ycpos $curline
-  ui_send_shared_pointer [expr $fw * ($xpos($cur_block) + $xcpos)] \
+  ui_send_shared_pointer [expr $fwu * ($xpos($cur_block) + $xcpos)] \
       [expr $fhu * ($ypos($cur_block) + $ycpos)]\
           $cur_col pen
   set_info_display
@@ -1316,25 +1454,23 @@ proc get_cur_tag {} {
 
 set maxid 1
 set old_block 0
+
 proc new_block {x y} {
-  global cur_block
-  global xpos ypos
-  global col cur_col
-  global font cur_font
-  global maxid
-  global exists
-  global fw fh
-  global old_block
+  global cur_block xpos ypos col cur_col font cur_font
+  global maxid exists fw fh old_block last_update
+  set last_update [clock seconds]
   if {$old_block==1} {
 #    puts oldblock
     set_info_display
     flash_block $cur_block
+	ui_active_block $cur_block
     set old_block 0
     return
   }
   set y [expr $y / $fh]
   set x [expr $x / $fw]
   set cur_block [ui_generate_id]
+  ui_active_block $cur_block
   set cur_line 0
   set xpos($cur_block) $x
   set ypos($cur_block) $y
@@ -1375,12 +1511,14 @@ proc flash_block {id} {
 pack append .m.l.txt \
     .m.l.txt.sb {right filly} \
     .m.l.txt.c {left fill expand}
-
+	
 
 if {$part_vis==1} {
     pack .m.l.ss -side right -fill both 
 }
-pack .m.l.txt -side left -fill both -expand true
+pack .m.l.hsb -side bottom -fill x -padx 2
+pack .m.l.txt -side top -fill both -expand true
+
 
 pack append .m \
      .m.cp {top fillx} \
@@ -1389,51 +1527,28 @@ pack append .m \
 
 pack append . .m {top fill expand}
 wm minsize . 600 200
-proc load_text {mode} {
-  global load_mode
-  set load_mode $mode
-  catch {destroy .load}
-  toplevel .load
-  wm title .load "nt: load file"
-  lognewwindow .load
-  if {[string compare $mode "plain"]==0} {
-      message .load.msg  -aspect 300 -text "Select a plain text file to load:"
-  } else {
-      message .load.msg  -aspect 300 -text "Select an nt structured file to load:"
-  }
-  frame .load.f
-  listbox .load.f.lb -width 40 -height 10 -relief sunken -yscroll ".load.f.sb set"
-  scrollbar .load.f.sb -command ".load.f.lb yview" \
-      -background [option get . background Mtool] \
-      -troughcolor [option get . scrollbarBackground Mtool]
-  pack .load.f.lb -side left
-  pack .load.f.sb -side right -fill y
-  .load.f.lb insert end ".."
-  foreach i [glob *] {
-    .load.f.lb insert end $i
-  }
-  label .load.l -text "File:"
-  entry .load.entry -width 40 -relief sunken
-  bind .load.entry <Return> {load_text_file $load_mode [.load.entry get]}
-  .load.entry insert 0 [pwd]
-  frame .load.bb -relief sunken
-  button .load.bb.load -text "Load" -command \
-    {load_text_file $load_mode [.load.entry get]}
-  button .load.bb.cancel -text "Cancel" -command "destroy .load"
-  bind .load.f.lb <1> { %W selection set [%W nearest %y];\
-      .load.entry delete 0 end;\
-      .load.entry insert 0 "[pwd]/[%W get [%W nearest %y]]"}
 
-  bind .load.f.lb <Double-Button-1> {load_text_file $load_mode [lindex [selection get] 0];break}
-  pack append .load.bb \
-      .load.bb.load {left expand fillx} \
-      .load.bb.cancel {left expand fillx}
-  pack append .load \
-      .load.msg {top fillx expand} \
-      .load.f  {top fill expand} \
-      .load.l {top fillx expand} \
-      .load.entry {top fillx expand} \
-      .load.bb {top fillx expand}
+#call it now to initialise the scrollbar...
+grow_canvas
+
+proc load_text {mode} {
+	global load_mode
+	set load_mode $mode
+
+	set types {
+		{{Text File} {.txt}  }
+	}
+	if {[string compare $mode "plain"]==0} {
+		set wtitle "Select a plain text file to load:"
+	} else {
+		set wtitle "Select an nte structured file to load:"
+	}
+	set filename [tk_getOpenFile -defaultextension .txt \
+		-filetypes $types \
+		-parent . -title $wtitle]
+
+	if {$filename == ""} return
+	load_text_file $load_mode $filename
 }
 
 proc load_text_file {mode file} {
@@ -1478,63 +1593,26 @@ proc load_text_file {mode file} {
 }
 
 proc save_text {mode} {
-  global save_mode
-  catch {destroy .save}
-  toplevel .save
-  wm title .save "nt: save file"
-  lognewwindow .save
-  set save_mode $mode
-  if {[string compare $mode "plain"]==0} {
-      message .save.msg  -aspect 300 -text "Select a plain text file to save:"
-  } else {
-      message .save.msg  -aspect 300 -text "Select a structured text file to save:"
-  }
-  frame .save.f
-  listbox .save.f.lb -width 40 -height 10 -relief sunken -yscroll ".save.f.sb set"
-  scrollbar .save.f.sb -command ".save.f.lb yview" \
-      -background [option get . background Mtool] \
-      -troughcolor [option get . scrollbarBackground Mtool]
-  pack .save.f.lb -side left
-  pack .save.f.sb -side right -fill y
-  .save.f.lb insert end ".."
-  foreach i [glob *] {
-    .save.f.lb insert end $i
-  }
-  label .save.l -text "File:"
-  entry .save.entry -width 40 -relief sunken
-  bind .save.entry <Return> {save_text_file $save_pptnts $save_mode [.save.entry get]}
-  .save.entry insert 0 [pwd]
-  global save_pptnts
-  set save_pptnts FALSE
-  if {[string compare $mode "plain"]==0} {
-      frame .save.f2 -borderwidth 0 -relief flat
-      checkbutton .save.f2.b -text "Save Participants" -variable save_pptnts\
-	  -onvalue TRUE -offvalue FALSE -borderwidth 1
-      pack .save.f2.b -side left -fill x -expand true
-  }
+	global save_mode
+	set save_mode $mode
 
-  frame .save.bb -relief sunken
-  button .save.bb.save -text "Save" -command \
-    {save_text_file $save_pptnts $save_mode [.save.entry get]}
-  button .save.bb.cancel -text "Cancel" -command "destroy .save"
-  bind .save.f.lb <1> {catch {%W selection set [%W nearest %y];\
-      .save.entry delete 0 end;\
-	  .save.entry insert 0 "[pwd]/[%W get [%W nearest %y]]"} }
+	set types {
+		{{Text File} {.txt}  }
+	}
+	if {[string compare $mode "plain"]==0} {
+		set wtitle "Select a plain text file to save:"
+	} else {
+		set wtitle "Select an nte structured file to save:"
+	}
+	set filename [tk_getSaveFile -defaultextension .txt \
+		-filetypes $types \
+		-parent . -title $wtitle]
 
-  bind .save.f.lb <Double-Button-1> {save_text_file $save_pptnts $save_mode [lindex [selection get] 0];break}
-  pack append .save.bb \
-      .save.bb.save {left expand fillx} \
-      .save.bb.cancel {left expand fillx}
-  pack append .save \
-      .save.msg {top fillx expand} \
-      .save.f {top fill expand}
-  pack .save.l -side top -fill x -expand true
-  pack .save.entry -side top -fill x -expand true
-  if {[string compare $mode "plain"]==0} {
-      pack .save.f2 -side top -fill x -expand true
-  }
-  pack .save.bb -side top -fill x -expand true
-  
+	if {$filename == ""} return
+
+	set save_pptnts [tk_dialog ".dialog" "save file" "save participant information?" "" 1 "Yes" "No" ]
+
+	save_text_file $save_pptnts $save_mode $filename 
 }
 
 
@@ -1582,6 +1660,16 @@ proc add_participant {c} {
     .m.l.ss.f.lb tag bind "l$c_str" <1> "show_participant_details %y"
 #    .m.l.ss.f.lb configure -state disabled
 }
+proc update_participant_details {pos y} {
+	set details [get_participant_by_number [expr $pos -1]]
+	if {$details==""} return
+	.pt$pos.f.l4 configure -text "Packets received: [lindex $details 5]"
+	after 1000 "participant_timer $pos $y"
+}
+
+proc participant_timer {pos y} {
+	if {[winfo exists .pt$pos]} {update_participant_details $pos $y}
+}
 
 proc show_participant_details {y} {
     global col
@@ -1599,14 +1687,18 @@ proc show_participant_details {y} {
     pack .pt$pos.f.l2 -side top -anchor nw
     label .pt$pos.f.l3 -text "Platform: [lindex $details 2]"
     pack .pt$pos.f.l3 -side top -anchor nw
+	label .pt$pos.f.l4 -text "Packets received: [lindex $details 5]"
+    pack .pt$pos.f.l4 -side top -anchor nw
     frame .pt$pos.f.f -borderwidth 0
     pack .pt$pos.f.f -side top -fill x -expand true
     label .pt$pos.f.f.l1 -text "Current colour:"
     pack .pt$pos.f.f.l1 -side left
     label .pt$pos.f.f.l2 -text "        " -background $col([lindex $details 3])
     pack .pt$pos.f.f.l2 -side right -fill x -expand true
+
     button .pt$pos.f.dismiss -text "Dismiss" -command "destroy .pt$pos"
     pack .pt$pos.f.dismiss -side top -fill x -expand true
+	after 1000 "participant_timer $pos $y"
 }
 proc rm_participant {n} {
 #    .m.l.ss.f.lb configure -state normal
@@ -1755,13 +1847,15 @@ proc create_shared_pointer {id name x y pcol style} {
 }
 
 proc move_my_shared_pointer {x y} {
-    global cur_col fhu fh
+    global cur_col fhu fh fwu fw
     
 	#((ADDED LINES - JIM))
 	set newy [expr $y*$fhu]
 	set newy [expr $newy/$fh]  
 	
-	move_shared_pointer myid me $x $newy $cur_col arrow
+	set newx [expr $x*$fwu]
+	set newx [expr $newx/$fw]  
+	move_shared_pointer myid me $newx $newy $cur_col arrow
 	#changes made by jim, to transmit 'unix friendly' pointer.
 	#replaced next line and added two extra lines above
 	
@@ -1769,14 +1863,14 @@ proc move_my_shared_pointer {x y} {
 	
 	
 	
-	ui_send_shared_pointer $x $newy $cur_col arrow
+	ui_send_shared_pointer $newx $newy $cur_col arrow
 }
 
 # pointer_style - used to store the last style of shared pointer drawn.
 set pointer_style "none"
 
 proc move_shared_pointer {id name x y pcol style} {
-    global pxpos pypos fhu fh pointer_style
+    global pxpos pypos fhu fh fwu fw pointer_style
     
 	
 	if {($x==65535)&&($y==65535)} {
@@ -1789,7 +1883,8 @@ proc move_shared_pointer {id name x y pcol style} {
 	# convert pointer y co-ords- this was setup for windows
 		set newy [expr $y*$fh]
 		set y [expr $newy/$fhu]
-
+		set newx [expr $x*$fw]
+		set x [expr $newx/$fwu]
 
     indicate_activity $id $y $pcol
     
@@ -1851,13 +1946,13 @@ global xpos ypos fw fh font col tcl_platform
 	    [.m.l.txt.c create text [expr $x * $fw] [expr $y * $fh] -anchor nw\
 	     -fill $col($coli) -font $font($fonti)]
         .m.l.txt.c bind $id <1> \
-            "set_cur_block $id %x \[expr %y + \[set canvypos\]\]"
+            "set_cur_block $id \[expr %x + \[set canvxpos\]\] \[expr %y + \[set canvypos\]\]"
         if {$tcl_platform(platform) == "unix"} {
 	    .m.l.txt.c bind $id <2> \
-		"startmoveblock $id %x \[expr %y + \[set canvypos\]\]"
+		"startmoveblock $id \[expr %x + \[set canvxpos\]\] \[expr %y + \[set canvypos\]\]"
 	} else {
 	    .m.l.txt.c bind $id <Control-ButtonPress-1> \
-		"startmoveblock $id %x \[expr %y + \[set canvypos\]\]"
+		"startmoveblock $id \[expr %x + \[set canvxpos\]\] \[expr %y + \[set canvypos\]\]"
 	}
 	set xpos($id) $x
 	set ypos($id) $y
@@ -1908,3 +2003,35 @@ proc enable_encryption {key} {
     }
 }
 grow_canvas
+
+set error_display 0
+set error_string "Previous versions of NTE are in use\
+	- to prevent instabilty please advise participants to upgrade."
+
+proc remove_error {} {
+	global error_display
+	destroy .m.in.error
+	pack .m.in.l .m.in.bid -side left
+	pack  .m.in.lck -side left -ipadx 10
+	pack .m.in.cl .m.in.cid  \
+		.m.in.ml .m.in.mid  \
+		.m.in.mtl .m.in.mt -side left
+	set error_display 1
+}
+
+proc upgrade_error {} {
+	global error_display error_string
+	if {$error_display==1} {return}
+	label .m.in.error -text $error_string -foreground darkred
+	pack forget .m.in.l .m.in.bid .m.in.lck \
+		.m.in.cl .m.in.cid \
+		.m.in.ml .m.in.mid \
+		.m.in.mtl .m.in.mt
+	pack propagate .m.in 0
+	pack .m.in.error -side left
+	after 700 {.m.in.error configure -text ""}
+	after 1400 {.m.in.error configure -text $error_string}
+	after 2100 {.m.in.error configure -text ""}
+	after 2800 {.m.in.error configure -text $error_string}
+	after 20000 {remove_error}
+}
