@@ -33,13 +33,15 @@
 #include "prototypes.h"
 #include <stdio.h>
 #include <string.h>
-#include <tcl.h>
-#include <tk.h>
+#include "tcl.h"
+#include "tk.h"
 
 #define NEW_SOLUTION
 extern block *find_block_by_id();
 extern line *find_line_by_id();
 extern line *find_last_line();
+extern block *active_block;
+int minimise_updates=1;
 /*extern block *init_block(); */
 extern line *init_line();
 extern void redisplay();
@@ -47,15 +49,16 @@ extern typeface tf;
 extern page *p;
 extern Tcl_Interp *interp;
 extern u_int16 lkey;
-extern struct in_addr ipaddr;
+
 extern void debug(const char *fmt,...);
 extern queue sendq;		/* transmission queue */
-extern int tx_fd;		/* transmission socket */
 extern struct timeval rtx_time;
 extern MAX_DELETED_BLOCKS;
 extern block *last_deleted_block[];	/* pointers to last 20 deleted blocks */
 extern no_of_del_blocks;
 
+extern u_int32 SrcID;
+
 int deal_with_page_checksum();
 
 int new_data = 0;
@@ -78,9 +81,8 @@ int seen_page = FALSE;
 /*parse_line: called when we have a LINEMSG received from the net   */
 /*            this fn decides what to do with the received line     */
 /********************************************************************/
-int parse_line(nl, t)
+int parse_line(nl)
 netline *nl;
-struct timeval *t;
 {
     block *b;
     line *l;
@@ -91,7 +93,9 @@ struct timeval *t;
     char nextid[ID_LEN];
     user_data modifier;
     int no_of_lines;
+    struct timeval t;
 
+    ntohtime(&(nl->lastmod), &t);
     ntohid(&(nl->blockid), blockid);
     ntohid(&(nl->lineid), lineid);
     ntohuser(&(nl->modifier), &modifier);
@@ -132,7 +136,7 @@ struct timeval *t;
 			       NORMAL,
 			       nl->xpos,
 			       ntohs(nl->ypos),
-			       t,
+			       &t,
 			       prev,
 			       NULL,
 			       &originator,
@@ -204,10 +208,10 @@ struct timeval *t;
 	    }
 	}
 
-	l = init_line(lineid, ntohs(nl->linenum), t, prevl, previd, nextl, nextid);	/* create line */
+	l = init_line(lineid, ntohs(nl->linenum), &t, prevl, previd, nextl, nextid);	/* create line */
 
 
-	add_to_recent_list(lineid, t, LINEMSG);		/* add line to the recent list */
+	add_to_recent_list(lineid, &t, LINEMSG);		/* add line to the recent list */
 
 	if ((prevl != NULL) && (strcmp(l->lineid, prevl->nextid) != 0)) {
 	    /* if there are missing lines before this line */
@@ -251,11 +255,12 @@ struct timeval *t;
 
 	l->no_of_chars = nl->no_of_chars;	/* copy line length */
 	l->block = b;		/* set block */
-	if (newer(t, &(b->last_mod)))	/* copy latest modification times for */
-	    time_copy(t, &(b->last_mod));	/*  block and page */
-	if (newer(t, &(p->last_mod)))
-	    time_copy(t, &(p->last_mod));
+	if (newer(&t, &(b->last_mod)))	/* copy latest modification times for */
+	    time_copy(&t, &(b->last_mod));	/*  block and page */
+	if (newer(&t, &(p->last_mod)))
+	    time_copy(&t, &(p->last_mod));
 	user_copy(&modifier, &(b->modifier));	/* update modifier */
+	user_copy(&modifier, &(l->modifier));
 
 	if ((b->first_line == NULL) || (prevl == NULL)) {
 	    b->first_line = l;
@@ -305,17 +310,20 @@ struct timeval *t;
 	if (b->status == DELETED)
 	    debug("deleted block.\n");
 
-	if (b->status != DELETED)
-	    delayed_redisplay_block(b);
+	if (b->status != DELETED) {
+			if ((active_block!=b) && (minimise_updates==1)) delayed_redisplay_block(b);
+				else redisplay_block(b);
+	}
 
     } else {			/* we have seen this line before. */
 
 	debug("got a modified line\n");
 	debug("b->no_of lines=%d\n", b->no_of_lines);
-	if (newer(t, &(l->last_mod)) || ((t->tv_sec == (&l->last_mod)->tv_sec) && ((&l->last_mod)->tv_usec == t->tv_usec))) {	/* if the received line is newer than our version of it */
-	    line *nextl;
+
+	if(newer(&t, &(l->last_mod))) {
+		line *nextl;
 	    line *prevl;
-	    debug("nt=%ld,%ld  ot=%ld,%ld\n", t->tv_sec, t->tv_usec, (l->last_mod).tv_sec, (l->last_mod).tv_usec);
+	    debug("nt=%ld,%ld  ot=%ld,%ld\n", &t.tv_sec, &t.tv_usec, (&l->last_mod)->tv_sec, (&l->last_mod)->tv_usec);
 	    if (is_in_missing_list(lineid))
 		rm_from_missing_list(lineid);	/* remove the line from the missing list */
 
@@ -323,9 +331,9 @@ struct timeval *t;
 	    debug("data:%s\n", l->line_data);
 	    l->no_of_chars = nl->no_of_chars;	/* update line length */
 	    debug("line has %d (%d) chars\n", l->no_of_chars, strlen(l->line_data));
-	    time_copy(t, &(l->last_mod));	/* update line, block and page update times */
-	    time_copy(t, &(b->last_mod));
-	    time_copy(t, &(p->last_mod));
+	    time_copy(&t, &(l->last_mod));	/* update line, block and page update times */
+	    time_copy(&t, &(b->last_mod));
+	    time_copy(&t, &(p->last_mod));
 	    user_copy(&(modifier), &(l->modifier));	/* update line and block modifiers */
 	    user_copy(&(modifier), &(b->modifier));
 
@@ -368,7 +376,7 @@ struct timeval *t;
 		debug("zeroing nextl->missing_lines\n");
 		nextl->missing_lines = 0;	/* tell next line to forget about lines it */
 	    }			/*   thinks are missing */
-	    add_to_recent_list(lineid, t, LINEMSG);	/* add this line to the list */
+	    add_to_recent_list(lineid, &t, LINEMSG);	/* add this line to the list */
 	    /*  of recent alterations. */
 	    if ((b->xpos != nl->xpos) || (b->ypos != ntohs(nl->ypos))) {	/* if block has moved !** */
 		/*block has moved */
@@ -396,9 +404,10 @@ struct timeval *t;
 		delete_line(l, b);
 	    }
 	    /*now need to force the update on screen */
-	    if (b->status != DELETED)
-		redisplay_block(b);
-
+		if (b->status != DELETED) {
+			if ((active_block!=b) && (minimise_updates==1)) delayed_redisplay_block(b);
+				else redisplay_block(b);
+		}
 
 
 	} else {		/* received line is older than ours? */
@@ -408,7 +417,7 @@ struct timeval *t;
 	    /* discarded here. */
 
 	    debug("must be an old packet - ignoring it\n");
-	    debug("nt=%ld,%ld  ot=%ld,%ld\n", t->tv_sec, t->tv_usec, l->last_mod.tv_sec, l->last_mod.tv_usec);
+	    debug("nt=%ld,%ld  ot=%ld,%ld\n", t.tv_sec, t.tv_usec, l->last_mod.tv_sec, l->last_mod.tv_usec);
 	    debug("data: >>%s<<\n", l->line_data);
 	    debug("id: %s\n", l->lineid);
 	    if (is_in_missing_list(lineid))
@@ -435,9 +444,8 @@ struct timeval *t;
 /*parse_block: called when we have a BLOCKMSG received from the net */
 /*            this fn decides what to do with the received block    */
 /********************************************************************/
-int parse_block(nb, t)
+int parse_block(nb)
 netblock *nb;
-struct timeval *t;
 {
     int code;
     static char str[100000];
@@ -449,6 +457,9 @@ struct timeval *t;
     char nextid[ID_LEN];
     user_data originator;
     user_data modifier;
+    struct timeval t;
+
+    ntohtime(&(nb->lastmod), &t);
     ntohid(&(nb->blockid), blockid);
     ntohid(&(nb->previd), previd);
     ntohid(&(nb->nextid), nextid);
@@ -540,7 +551,7 @@ struct timeval *t;
 			       nb->status,
 			       nb->xpos,
 			       ntohs(nb->ypos),
-			       t,
+			       &t,
 			       prev,
 			       next,
 			       &(originator),
@@ -553,7 +564,7 @@ struct timeval *t;
 	    p->first_block = new_block;
 
 	p->last_block = new_block;
-	add_to_recent_list(blockid, t, BLOCKMSG);
+	add_to_recent_list(blockid, &t, BLOCKMSG);
 
 	sprintf(command, "set_block_posn %s %d %d",
 		blockid, nb->xpos, ntohs(nb->ypos));
@@ -571,8 +582,8 @@ struct timeval *t;
 	    p->last_block = new_block;
 	}
 #endif
-	if (newer(t, &(p->last_mod)))
-	    time_copy(t, &(p->last_mod));
+	if (newer(&t, &(p->last_mod)))
+	    time_copy(&t, &(p->last_mod));
 
 	/* now deal with receiving new blocks, that were already deleted.. */
 	if (new_block->status == DELETED) {
@@ -594,14 +605,14 @@ struct timeval *t;
 
 		code = Tcl_Eval(interp, command);	/* display block on screen display */
 
-		sprintf(command, "try_to_delete_block %s", new_block->blockid);
+		sprintf(command, "set_delete_status %s", new_block->blockid);
 		Tcl_Eval(interp, command);
 	    }			/* end else */
 
 	}			/* end if deleted */
     } else {
 	/*we've seen this block before */
-	if (newer(t, &(b->last_mod)) || is_a_null_name(&(b->originator)) || ((t->tv_sec == b->last_mod.tv_sec) && (t->tv_usec == b->last_mod.tv_usec))) {
+	if (newer(&t, &(b->last_mod)) || is_a_null_name(&(b->originator)) || ((t.tv_sec == b->last_mod.tv_sec) && (t.tv_usec == b->last_mod.tv_usec))) {
 	    debug("got a modified block\n");
 	    b->new_data = 1;
 
@@ -679,10 +690,10 @@ struct timeval *t;
 			   to set their modification time to the same
 			   value, and it must be greater than the person
 			   who thinks this block still exists */
-			t->tv_usec++;
-			if (t->tv_usec >= 1000000) {
-			    t->tv_sec++;
-			    t->tv_usec = 0;
+			t.tv_usec++;
+			if (t.tv_usec >= 1000000) {
+			    t.tv_sec++;
+			    t.tv_usec = 0;
 			}
 		    }
 		    break;
@@ -695,22 +706,22 @@ struct timeval *t;
 			debug("nb->xpos =%d \n", nb->xpos);
 			debug("received a deleted block\n");
 
-			sprintf(command, "try_to_delete_block %s", blockid);
+			sprintf(command, "set_delete_status %s", blockid);
 			Tcl_Eval(interp, command);
-
-		    }		/* end case deleted */
+		    }
+			b->status = DELETED;
 		    break;
 		}
 	    }
-	    time_copy(t, &(b->last_mod));
-	    time_copy(t, &(p->last_mod));
-	    add_to_recent_list(blockid, t, BLOCKMSG);
+	    time_copy(&t, &(b->last_mod));
+	    time_copy(&t, &(p->last_mod));
+	    add_to_recent_list(blockid, &t, BLOCKMSG);
 	} else {
 	    debug("got an old block\n");
-	    debug("nt=%lu,%lu  ot=%lu,%lu\n", t->tv_sec, t->tv_usec, b->last_mod.tv_sec, b->last_mod.tv_usec);
+	    debug("nt=%lu,%lu  ot=%lu,%lu\n", t.tv_sec, t.tv_usec, b->last_mod.tv_sec, b->last_mod.tv_usec);
 	    if ((nb->status == DELETED) & (b->status != DELETED)) {
 		b->status = DELETED;
-		sprintf(command, "delete_block %s", blockid);
+		sprintf(command, "set_delete_status %s", blockid);
 		Tcl_Eval(interp, command);
 	    }
 	}
@@ -725,16 +736,17 @@ u_int16 masks[17] =
 {0x0, 0x1, 0x3, 0x7, 0xf, 0x1f, 0x3f, 0x7f, 0xff,
  0x1ff, 0x3ff, 0x7ff, 0xfff, 0x1fff, 0x3fff, 0x7fff, 0xffff};
 
-int parse_rtx_advert(nrtxa, remote_addr, t)
+int parse_rtx_advert(nrtxa, remote_addr)
 netrtxadvert *nrtxa;
-struct in_addr *remote_addr;
-struct timeval *t;
+u_int32 remote_addr;
 {
     struct timeval last_mod;
     struct timeval cur_time;
+    struct timeval t;
     int received_sum = RECEIVED_SUM;
 
-    seen_another_sliding_key(t);
+    ntohtime(&(nrtxa->lastmod), &t);
+    seen_another_sliding_key(&t);
     get_cur_time(&rtx_time);
     debug("Received : Key %d, Mask %d :", (ntohs(nrtxa->key)), (masks[nrtxa->mask]));
 
@@ -779,29 +791,28 @@ struct timeval *t;
     } else if (number_missing() > 0) {
 	debug("sending a specific rtx request\n");
 	send_specific_rtx_request(remote_addr);
-    } else if (newer(t, &last_mod)) {
+    } else if (newer(&t, &last_mod)) {
 	/*we need a retransmission */
 	debug("Asking for rtx after time t (t: %lu,%lu, lm: %lu,%lu\n",
-	      t->tv_sec, t->tv_usec, last_mod.tv_sec, last_mod.tv_usec);
+	      t.tv_sec, t.tv_usec, last_mod.tv_sec, last_mod.tv_usec);
 	send_rtx_request(remote_addr, &last_mod);
     }
     return 0;
 }
 
-int parse_rtx_request(nrtxr, t)
+int parse_rtx_request(nrtxr)
 netrtxrequest *nrtxr;
-struct timeval *t;
 {
     debug("Got an RTX REQUEST -");
-    if ((ntohl(nrtxr->source.s_addr)) == (ipaddr.s_addr)) {
-	debug(" addressed to me!\n");
-	send_retransmissions(t);
-    } else {
-	debug(" not addressed to me!\n");
-	/* by design, we should be using the knowledge that someone else has requested */
-	/* several items to ensure that we do not immediately request the same items. */
 
-    }				/* end else */
+    if (ntohl(nrtxr->source_uid)==SrcID) {
+        debug(" addressed to me!\n");
+        send_retransmissions(&(nrtxr->lastmod));
+    } else {
+        debug(" not addressed to me!\n");
+        /* by design, we should be using the knowledge that someone else has requested */
+        /* several items to ensure that we do not immediately request the same items. */
+    }
     return 0;
 }
 
@@ -811,21 +822,17 @@ netrtxspecificrequest *nrtxr;
     char id[ID_LEN];
     debug("Got an RTX SPECIFIC REQUEST - ");
 
-    if (ntohl(nrtxr->source.s_addr) == (ipaddr.s_addr)) {	/* if we're being asked for information */
-	u_int i;
-
-	debug(" addressed to me!\n");
-	for (i = 0; i < ntohl(nrtxr->no_of_items); i++) {	/* for each element in retransmission list */
-	    ntohid(&(nrtxr->ids[i]), id);
-	    debug("request for %s\n", id);
-
-	    send_specific_retransmission(id);	/* queue that element for transmission */
-	}
+    if (ntohl(nrtxr->source_uid)==SrcID) {
+        u_int i;
+        debug(" addressed to me!\n");
+        for (i = 0; i < ntohl(nrtxr->no_of_items); i++) {	/* for each element in retransmission list */
+            ntohid(&(nrtxr->ids[i]), id);
+	        debug("request for %s\n", id);
+            send_specific_retransmission(id);	/* queue that element for transmission */
+        }
     } else {
-
-	debug(" addressed to someone else");
-
-/*      printf("It was addressed to %s\n", inet_ntoa(nrtxr->source.s_addr)); */
+        debug(" addressed to someone else");
+        /*printf("It was addressed to %s\n", inet_ntoa(nrtxr->source.s_addr)); */
     }
     return 0;
 }
@@ -834,11 +841,13 @@ int parse_page_request(netpagerequest * nrtxr)
 {
     struct timeval t;
     debug("Got a PAGE_REQUEST ");
-    if (ntohl(nrtxr->source.s_addr) == (ipaddr.s_addr)) {
-	debug(" - addressed to me!\n");
-	t.tv_sec = 0;
-	t.tv_usec = 0;
-	send_retransmissions(&t);	/* retransmit everything (i.e. after time=0) */
+
+    if (ntohl(nrtxr->source_uid)==SrcID) {
+        debug(" - addressed to me!\n");
+        t.tv_sec = 0;
+	    t.tv_usec = 0;
+        /* retransmit everything (i.e. after time=0) */
+	    send_retransmissions(&t);
     }
     debug(" - not addressed to me\n");
     seen_page = TRUE;
@@ -868,6 +877,7 @@ int parse_pointer(netpointer * np)
 {
     char str[256];
     char id[ID_LEN];
+
     ntohid(&(np->id), id);
     if (ntohl(np->x) != DELETE_POINTER) {
 	debug("move_shared_pointer %s %s %d %d %d %s\n", id, np->username,
@@ -964,7 +974,7 @@ int parse_recent_list(netrecentadvert * nra)
 	    }
 	} else {		/* the recent list contains an element with type != (line or block) */
 	    ntohid(&(nra->entries[i].id), id);
-	    fprintf(stderr, "Got an unknown object type %d in recent advert from %s\n", nra->entries[i].type, id);
+	    debug("Got an unknown object type %d in recent advert from %s\n", nra->entries[i].type, id);
 	}
     }
     return 0;
@@ -997,7 +1007,7 @@ int deal_with_page_checksum()
 	   the current site. We are assuming here that the current site has more info 
 	   than us.. */
 
-	send_blockinfo_request2(&(store_npi.source));
+	send_blockinfo_request2(store_npi.source_uid);
 	debug("sending block info request for all blocks\n");
 
     } else {
@@ -1027,7 +1037,7 @@ int parse_page_info(netpageinfo * npi)
     /* store checksum and source address received */
 
     strncpy(store_npi.checksum, npi->checksum, 18);
-    memcpy(&(store_npi.source), &(npi->source), sizeof(struct in_addr));
+    store_npi.source_uid = ntohl(npi->source_uid);
 
     sum_received = RECEIVED_SUM;
 
@@ -1080,13 +1090,10 @@ int parse_blockinfo_request(netblockinforequest * nbir)
     char id[ID_LEN];
 
     debug("parsing block info request packet\n");
-    debug("received %d\n", ( /*ntohl( */ nbir->target.s_addr /*) */ ));
-    debug("our addr %d\n", (ipaddr.s_addr));
 
-
-    if (( /*ntohl( */ nbir->target.s_addr /*) */ ) != (ipaddr.s_addr)) {
-	debug("block info request not directed to this site\n");
-	return 0;		/* if the request is not directed towards this site.. */
+    if (ntohl(nbir->source_uid)!=SrcID) {
+        debug("block info request not directed to this site\n");
+        return 0;		/* if the request is not directed towards this site.. */
     }
     /* someone has requested that we send blockinfo only for block with id nbir->blockid. */
     ntohid(&(nbir->blockid), id);
@@ -1145,7 +1152,7 @@ int parse_blockinfo(netblockinfo * nbi)
     if (b == NULL) {
 	/* if we haven't got this block, then just request it immediately */
 
-	send_block_rtx_request(id, &(nbi->source));
+	send_block_rtx_request(id, nbi->source_uid);
     }
     /* end if */
     else {
@@ -1153,7 +1160,7 @@ int parse_blockinfo(netblockinfo * nbi)
 	    /* re-request transmission of this block (including all it's lines), */
 	    /* send a block_rtx_request message */
 	    debug("We have received a block information packet with a different checksum\n");
-	    send_block_rtx_request(id, &(nbi->source));
+	    send_block_rtx_request(id, nbi->source_uid);
 
 	    /* ** NOTE ** - if there are problems with the new updates conflicting with our */
 	    /*  version of this block, we should perhaps delete our local store in some way, */
@@ -1174,10 +1181,10 @@ int parse_blockrtx_request(netblockrtxrequest * nbrr)
     char id[ID_LEN];
     /* retransmit the block referred to by nbrr and then retransmit all of it's lines too */
 
-    if ( /*(ntohl */ (nbrr->target.s_addr /*) */ ) != (ipaddr.s_addr)) {
-	debug("received a block rtx request, addressed to another site\n");
-	return 0;		/* if this is not the site that retransmission is */
-	/* requested from. */
+    if (ntohl(nbrr->target_uid)!=SrcID) {
+        debug("received a block rtx request, addressed to another site\n");
+        return 0;   /* if this is not the site that retransmission is
+	                   requested from. */
     }
     debug("parsing a block rtx request packet\n");
     /* someone has requested that we send blockinfo only for block with id nbir->blockid. */
@@ -1193,11 +1200,11 @@ int parse_blockrtx_request(netblockrtxrequest * nbrr)
 	} else {
 	    line *l;
 	    /* retransmit block with idno id, and all it's lines */
-	    queue_packet_for_sending(tx_fd, b, b->blockid, &sendq, BLOCKMSG);
+	    queue_packet_for_sending(b, b->blockid, &sendq, BLOCKMSG);
 	    l = b->first_line;
 
 	    while (l != NULL) {
-		queue_packet_for_sending(tx_fd, l, l->lineid, &sendq, LINEMSG);
+		queue_packet_for_sending(l, l->lineid, &sendq, LINEMSG);
 		l = l->next_line;
 	    }			/* end while */
 
