@@ -31,13 +31,12 @@
  * SUCH DAMAGE.
  */
 #include "prototypes.h"
-#include <tcl.h>
-#include <tk.h>
+#include "tcl.h"
+#include "tk.h"
 #include <string.h>
 
 extern void debug(const char *fmt,...);		/* output to standard error stream command */
 extern void time_copy();
-extern int tx_fd;
 queue sendq;
 queue rtxq;
 extern sliding_key skey;
@@ -47,9 +46,6 @@ extern Tcl_Interp *interp;
 extern u_int8 cur_col;
 extern char sessionname[];
 
-
-extern struct in_addr ipaddr;	/* variable storing our local ip address */
-
 extern page *p;			/* variable that refers to the main page. */
 
 extern char vanityname[];
@@ -67,7 +63,7 @@ extern struct timeval rtx_time;
 int send_queued_packet();
 extern unsigned int subtract_time();
 char *get_missing_id();
-
+extern u_int32 SrcID;
 extern int checksum_active;
 
 const int NORMAL_S_MESSAGE = 0;	/* normal session message type */
@@ -76,12 +72,16 @@ const int COLOUR_CHANGE = 1;	/* session message to signal user */
 
 int s_message_type;
 
-void init_header(ntheader * header, unsigned char type, struct timeval *t)
+void init_header(ntheader * header, unsigned char type)
 {
+    struct timeval t;
+    struct timezone tzp;
+    gettimeofday(&t, &tzp);
     header->checkbyte = (unsigned char) NT_PROTOCOL_CHECKBYTE;
     header->version_no = (unsigned char) CURRENT_PROTOCOL_VERSION;
-    htontime(t, &(header->timestamp));
+    htontime(&t, &(header->timestamp));
     header->msgtype = type;
+	header->SrcID = htonl (SrcID) ;
 }
 
 void init_queue(q)
@@ -93,8 +93,7 @@ queue *q;
     q->queue_size = 0;
 }
 
-int send_line(fd, l, b)
-int fd;
+int send_line(l, b)
 line *l;
 block *b;
 {
@@ -102,7 +101,8 @@ block *b;
     netline nl;
     int len;
 
-    init_header(&header, LINEMSG, &(l->last_mod));
+    init_header(&header, LINEMSG);
+    htontime(&(l->last_mod), &(nl.lastmod));
     htonid(l->lineid, &(nl.lineid));
     if (b != NULL) {
 	htonid(b->blockid, &(nl.blockid));
@@ -137,17 +137,17 @@ block *b;
     len = sizeof(netline) + (nl.no_of_chars + 1) - MAX_LINE_LEN;
     debug("Sending: >>%s<<\n", l->line_data);
     debug("line id: >>%s<<\n", l->lineid);
-    xsocksend(fd, &header, (u_char *) & nl, len);
+    xsocksend(&header, (u_char *) & nl, len);
     return 0;
 }
 
-int send_block(fd, b)
-int fd;
+int send_block(b)
 block *b;
 {
     ntheader header;
     netblock nb;
-    init_header(&header, BLOCKMSG, &(b->last_mod));
+    init_header(&header, BLOCKMSG);
+    htontime(&(b->last_mod), &(nb.lastmod));
     htonid(b->blockid, &(nb.blockid));
 
     if (b->prev_block != NULL)
@@ -173,29 +173,26 @@ block *b;
     nb.ypos = htons(b->ypos);
     nb.no_of_lines = htons(b->no_of_lines);
     debug("sending block id: >>%s<<\n", b->blockid);
-    xsocksend(fd, &header, (u_char *) & nb, sizeof(netblock));
+    xsocksend(&header, (u_char *) & nb, sizeof(netblock));
     return 0;
 }
 
 
 
-int send_pointer(fd, p)
-int fd;
+int send_pointer(p)
 pointer *p;
 {
     ntheader header;
     netpointer np;
-    struct timeval t;
-    get_cur_time(&t);
-    debug("secs: %lu, usecs: %lu\n", t.tv_sec, t.tv_usec);
-    init_header(&header, POINTERMSG, &t);
+
+    init_header(&header, POINTERMSG);
     htonid(p->id, &(np.id));
     strncpy(np.username, p->username, MAX_USER_NAME);
     np.x = htonl(p->x);
     np.y = htonl(p->y);
     np.col = p->col;
     strncpy(np.style, p->style, STYLE_LEN);
-    xsocksend(fd, &header, (u_char *) & np, sizeof(netpointer));
+    xsocksend(&header, (u_char *) & np, sizeof(netpointer));
     return 0;
 }
 
@@ -274,7 +271,7 @@ queue *q;
 char *id;
 {
     int i, j;
-    fprintf(stderr, "******DELETE_FROM_QUEUE needs fixing\n");
+    debug("******DELETE_FROM_QUEUE needs fixing\n");
     for (i = 0; i < q->queue_size; i++) {
 	if (strncmp(q->qdata[i].id, id, ID_LEN) == 0) {
 	    for (j = i; j < q->queue_size - 1; j++) {
@@ -287,8 +284,7 @@ char *id;
 }
 
 
-void queue_packet_for_sending(fd, l, id, q, type)
-int fd;
+void queue_packet_for_sending(l, id, q, type)
 void *l;			/*Either a line or a block */
 char *id;
 queue *q;
@@ -341,21 +337,21 @@ int type;
 	    q->last_size = sizeof(ntheader) + sizeof(netline)
 		+ (((line *) l)->no_of_chars + 1) - MAX_LINE_LEN;
 	    init_rtx_token(&skey);
-	    send_line(fd, (line *) l, ((line *) l)->block);
+	    send_line((line *) l, ((line *) l)->block);
 	    break;
 	case UNKNOWNLINEMSG:
 	    strncpy(unknown_line.lineid, id, ID_LEN);
 	    unknown_line.linenum = UNKNOWN_LINE;
-	    send_line(tx_fd, &unknown_line, NULL);
+	    send_line(&unknown_line, NULL);
 	    break;
 	case BLOCKMSG:
 	    q->last_size = sizeof(ntheader) + sizeof(netblock);
 	    init_rtx_token(&skey);
-	    send_block(fd, (block *) l);
+	    send_block((block *) l);
 	    break;
 	case POINTERMSG:
 	    q->last_size = sizeof(ntheader) + sizeof(netpointer);
-	    send_pointer(fd, (pointer *) l);
+	    send_pointer((pointer *) l);
 	}
     }
 }
@@ -374,24 +370,24 @@ int send_queued_packet()
     l = get_from_queue(q, &type, unknown_line.lineid);
     if (l == NULL) {
 	/*this shouldn't ever happen */
-	fprintf(stderr, "NULL packet got into send queue\n");
+	debug("NULL packet got into send queue\n");
 	if (packets_queued(q) > 0)
 	    send_queued_packet();
 	return TCL_OK;
     }
     switch (type) {
     case LINEMSG:
-	send_line(tx_fd, (line *) l, ((line *) l)->block);
+	send_line((line *) l, ((line *) l)->block);
 	break;
     case UNKNOWNLINEMSG:
 	unknown_line.linenum = UNKNOWN_LINE;
-	send_line(tx_fd, &unknown_line, NULL);
+	send_line(&unknown_line, NULL);
 	break;
     case BLOCKMSG:
-	send_block(tx_fd, (block *) l);
+	send_block((block *) l);
 	break;
     case POINTERMSG:
-	send_pointer(tx_fd, (pointer *) l);
+	send_pointer((pointer *) l);
 	break;
     }
     if (packets_queued(q) > 0) {
@@ -408,29 +404,27 @@ int send_queued_packet()
     return TCL_OK;
 }
 
-void send_rtx_request(remote_addr, t)
-struct in_addr *remote_addr;
+void send_rtx_request(remote_id, t)
+u_int32 remote_id;
 struct timeval *t;
 {
     ntheader header;
     netrtxrequest request;
-    init_header(&header, RTX_REQUEST, t);
-    memcpy(&(request.source), remote_addr, sizeof(struct in_addr));
-    xsocksend(tx_fd, &header, (u_char *) & request, sizeof(netrtxrequest));
+    init_header(&header, RTX_REQUEST);
+    htontime(t, &(request.lastmod));
+    request.source_uid = htonl(remote_id);
+    xsocksend(&header, (u_char *) & request, sizeof(netrtxrequest));
 }
 
-void send_specific_rtx_request(remote_addr)
-struct in_addr *remote_addr;
+void send_specific_rtx_request(remote_id)
+u_int32 remote_id;
 {
     ntheader header;
-    struct timeval t;
-    struct timezone tzp;
     netrtxspecificrequest request;
     int i, no_missing;
 
-    gettimeofday(&t, &tzp);
-    init_header(&header, RTX_SPECIFIC_REQUEST, &t);
-    memcpy(&(request.source), remote_addr, sizeof(struct in_addr));
+    init_header(&header, RTX_SPECIFIC_REQUEST);
+    request.source_uid = htonl(remote_id);
     no_missing = number_missing();
     if (no_missing > MAX_NO_OF_REQUESTS)
 	no_missing = MAX_NO_OF_REQUESTS;
@@ -439,23 +433,20 @@ struct in_addr *remote_addr;
 	htonid(get_missing_id(i), &(request.ids[i]));
     }
     request.no_of_items = htonl(no_missing);
-    xsocksend(tx_fd, &header, (u_char *) & request, sizeof(struct in_addr) +
-	      sizeof(int) + (ID_LEN * no_missing));
+    xsocksend(&header, (u_char *) & request, sizeof(u_int32) +
+	      sizeof(int) + ((sizeof(netid)) * no_missing));
 
 }
 
-void send_page_request(remote_addr)
-struct in_addr *remote_addr;
+void send_page_request(remote_id)
+u_int32 remote_id;
 {
     ntheader header;
-    struct timeval t;
-    struct timezone tzp;
     netpagerequest request;
 
-    gettimeofday(&t, &tzp);
-    init_header(&header, PAGE_INFO_REQUEST, &t);
-    memcpy(&(request.source), remote_addr, sizeof(struct in_addr));
-    xsocksend(tx_fd, &header, (u_char *) & request, sizeof(netpagerequest));
+    init_header(&header, PAGE_INFO_REQUEST);
+    request.source_uid = htonl(remote_id);
+    xsocksend(&header, (u_char *) & request, sizeof(netpagerequest));
 
 }
 
@@ -473,13 +464,9 @@ void send_session_message()
     static netsession sm;
     static int first = 0;
     if (first == 0) {
-	/*no point in setting all this up every time */
-	struct timeval t;
-	struct timezone tzp;
 	char *str;
-	gettimeofday(&t, &tzp);
-	/*sure I'll think of a use for t eventually */
-	init_header(&header, SESSIONMSG, &t);
+
+	init_header(&header, SESSIONMSG);
 	htonuser(&me, &sm.ud);
 	if (strcmp(vanityname, "") == 0)
 	    sprintf(sm.textname, "%s@%s", me.username, hostname);
@@ -501,7 +488,7 @@ void send_session_message()
 
     debug("sending a session message\n");
 
-    xsocksend(tx_fd, &header, (u_char *) & sm, sizeof(user_data) +
+    xsocksend(&header, (u_char *) & sm, sizeof(user_data) +
 	      sizeof(u_int8) + (sm.textnamelen + 1));
 
     timeout_participants();
@@ -530,11 +517,7 @@ void send_session_message()
 
 	while (b != NULL) {
 	    debug("new_data = %d", b->new_data);
-	    if (b->new_data == 1) {
-		b->new_data = 0;
-		redisplay_block(b);
-
-	    }
+	    if (b->new_data == 1) redisplay_block(b);
 	    b = b->next_block;
 	}
 
@@ -570,11 +553,8 @@ void send_leave_message()
 {
     static ntheader header;
     static netsession sm;
-    struct timeval t;
-    struct timezone tzp;
 
-    gettimeofday(&t, &tzp);
-    init_header(&header, LEAVE_CONF, &t);
+    init_header(&header, LEAVE_CONF);
     htonuser(&me, &sm.ud);
     sprintf(sm.textname, "%s@%s", me.username, hostname);
 
@@ -582,7 +562,7 @@ void send_leave_message()
     /* leaving message bits. */
     debug("sending leave message with name:%s\n", sm.textname);
 
-    xsocksend(tx_fd, &header, (u_char *) & sm, sizeof(user_data) +
+    xsocksend(&header, (u_char *) & sm, sizeof(user_data) +
 	      sizeof(u_int8) + (sm.textnamelen + 1));
 
 }
@@ -593,109 +573,79 @@ void send_page_info()
 
     static ntheader header;
     static netpageinfo npi;
-    struct timeval t;
-    struct timezone tzp;
 
-    if (checksum_active == 0) {
-	return;
-    }
-    gettimeofday(&t, &tzp);	/* create the packet's header */
-    init_header(&header, PAGE_INFO, &t);
+    if (checksum_active == 0) return;
+
+    /* create the packet's header */
+    init_header(&header, PAGE_INFO);
 
     htontime(&(p->last_mod), &(npi.timestamp));
-
     calc_page_checksum(p);	/* re-calculate this page's checksum */
-
     strncpy(npi.checksum, p->checksum, 18);	/* set the packet's fields correctly */
-
-    memcpy(&(npi.source), &ipaddr, sizeof(struct in_addr));
-
+    npi.source_uid = htonl (SrcID);
     npi.checksum_version = CHECKSUM_ALGORITHM_VERSION;
-
     npi.no_of_blocks = htons(p->no_of_blocks);
-
     debug("sending page info packet\n");
 
     /* send the packet */
-    xsocksend(tx_fd, &header, (u_char *) & npi, sizeof(netpageinfo));
+    xsocksend(&header, (u_char *) & npi, sizeof(netpageinfo));
 
-}				/* end send_page_info packet */
+}
 
-void send_blockinfo_request(block * b, struct in_addr *target)
+void send_blockinfo_request(block * b, u_int32 target)
 {
     static ntheader header;
     static netblockinforequest nbir;
-    struct timeval t;
-    struct timezone tzp;
 
+    if (checksum_active == 0) return;
 
-    if (checksum_active == 0) {
-	return;
-    }
-    gettimeofday(&t, &tzp);
-    init_header(&header, BLOCK_INFO_REQUEST, &t);
-
-    memcpy(&(nbir.target), target, sizeof(struct in_addr));
-
+    init_header(&header, BLOCK_INFO_REQUEST);
+    nbir.source_uid = htonl (target);
     htonid(b->blockid, &(nbir.blockid));
 
-    xsocksend(tx_fd, &header, (u_char *) & nbir, sizeof(netblockinforequest));
+    xsocksend(&header, (u_char *) & nbir, sizeof(netblockinforequest));
     debug("sending block info request packet\n");
 
-}				/* end send_blockinfo_request */
+}
 
-void send_blockinfo_request2(struct in_addr *target)
-{				/* version 2 - requests transmission *//* of info on all blocks in the page */
+void send_blockinfo_request2(u_int32 target)
+{
+	/* version 2 - requests transmission */
+    /* of info on all blocks in the page */
     static ntheader header;
     static netblockinforequest2 nbir;
-    struct timeval t;
-    struct timezone tzp;
 
+    if (checksum_active == 0) return;
 
-    if (checksum_active == 0) {
-	return;
-    }
-    gettimeofday(&t, &tzp);
-    init_header(&header, BLOCK_INFO_REQUEST2, &t);
+    init_header(&header, BLOCK_INFO_REQUEST2);
+    nbir.target_uid = htonl(target);
 
-    memcpy(&(nbir.target), target, sizeof(struct in_addr));
-
-    xsocksend(tx_fd, &header, (u_char *) & nbir, sizeof(netblockinforequest2));
+    xsocksend(&header, (u_char *) & nbir, sizeof(netblockinforequest2));
     debug("sending block info request packet version 2\n");
-
-}				/* end send_blockinfo_request */
+}
 
 
 void send_blockinfo(block * b)
 {
     static ntheader header;	/* broadcast information about a block to everyone */
     static netblockinfo nbi;
-    struct timeval t;
-    struct timezone tzp;
 
-    if (checksum_active == 0) {
-	return;
-    }
-    gettimeofday(&t, &tzp);
-    init_header(&header, BLOCK_INFO, &t);
+    if (checksum_active == 0) return;
 
-    memcpy(&(nbi.source), &ipaddr, sizeof(struct in_addr));
+    init_header(&header, BLOCK_INFO);
 
+    nbi.source_uid = htonl (SrcID);
     nbi.checksum_version = CHECKSUM_ALGORITHM_VERSION;
-
     htonid(b->blockid, &nbi.blockid);
-
     strncpy(nbi.checksum, b->checksum, 18);
-
     nbi.no_of_lines = htons(b->no_of_lines);
-
-    xsocksend(tx_fd, &header, (u_char *) & nbi, sizeof(netblockinfo)	/*sizeof(struct in_addr) +
+    xsocksend(&header, (u_char *) & nbi, sizeof(netblockinfo)	/*sizeof(struct in_addr) +
 									   sizeof(u_int16) + (sizeof(u_char)*20) + sizeof(u_int16) + sizeof(netid) */ );
 
     debug("sending block info packet\n");
-}				/* end send_blockinfo */
+}
 
-void send_block_rtx_request(char id[ID_LEN], struct in_addr *source)
+void send_block_rtx_request(char id[ID_LEN], u_int32 source)
 {
     /* send a request to the site at address source to retransmit the block with idno==id, */
     /* as well as all of it's lines */
@@ -703,20 +653,13 @@ void send_block_rtx_request(char id[ID_LEN], struct in_addr *source)
 
     static ntheader header;
     static netblockrtxrequest nbrr;
-    struct timeval t;
-    struct timezone tzp;
 
-    if (checksum_active == 0) {
-	return;
-    }
-    gettimeofday(&t, &tzp);
-    init_header(&header, BLOCK_RTX_REQUEST, &t);
-
-    memcpy(&(nbrr.target), source, sizeof(struct in_addr));
+    if (checksum_active == 0) return;
 
+    init_header(&header, BLOCK_RTX_REQUEST);
+    nbrr.target_uid = htonl(source);
     htonid(id, &(nbrr.blockid));
-
-    xsocksend(tx_fd, &header, (u_char *) & nbrr, sizeof(netblockrtxrequest));
+    xsocksend(&header, (u_char *) & nbrr, sizeof(netblockrtxrequest));
 
     debug("sending block retransmission request\n");
-}				/* end send_block_rtx_request */
+}
