@@ -1,4 +1,4 @@
-#include <tcl.h>
+#include "tcl.h"
 
 int TkPlatformInit(Tcl_Interp * interp)
 {
