@@ -38,8 +38,8 @@
  */
 #include "prototypes.h"
 #include <stdio.h>
-#include <tcl.h>
-#include <tk.h>
+#include "tcl.h"
+#include "tk.h"
 #include <stdlib.h>
 #include <string.h>
 #include <sys/types.h>
@@ -47,15 +47,16 @@
 #include <time.h>
 
 extern int checksum_active;	/* used to store checksum activation status */
+extern int minimise_updates;
 extern page *p;
 extern user_data me;
 extern typeface tf;
 extern Tcl_Interp *interp;
-extern int tx_fd;
 extern queue sendq;
 extern int max_block_len;
 extern p_list plist;
 block *cur_block = NULL;
+block *active_block = NULL;
 block *clipboard_block = NULL;	/* pointer to block currently 'on the clipboard' */
 int clipboard_counter = 1;	/* used to count the copies being made of a block, so that
 				   subsequent pastings can be placed in descending positions */
@@ -127,11 +128,11 @@ int ui_new_block(ClientData dummy, Tcl_Interp * interp, int argc, char **argv)
     new_ln->block = new_blk;
 
     /* now we transmit the new block */
-    queue_packet_for_sending(tx_fd, new_blk, new_blk->blockid, &sendq, BLOCKMSG);
+    queue_packet_for_sending(new_blk, new_blk->blockid, &sendq, BLOCKMSG);
     add_to_recent_list(new_blk->blockid, &(new_blk->last_mod), BLOCKMSG);
 
     /* JIM CHANGE - SEND OUR NEW LINE AS WELL - THIS AVOIDS NULL POINTERS IN RECEIVING HOSTS.. */
-    queue_packet_for_sending(tx_fd, new_ln, new_ln->lineid, &sendq, LINEMSG);
+    queue_packet_for_sending(new_ln, new_ln->lineid, &sendq, LINEMSG);
     add_to_recent_list(new_ln->lineid, &(new_ln->last_mod), LINEMSG);
 
     if (new_blk == NULL) {
@@ -262,7 +263,7 @@ char **argv;			/* Argument strings. */
 
 
 
-    queue_packet_for_sending(tx_fd, cur_line, cur_line->lineid, &sendq, LINEMSG);
+    queue_packet_for_sending(cur_line, cur_line->lineid, &sendq, LINEMSG);
     add_to_recent_list(cur_line->lineid, &t, LINEMSG);
 
     return TCL_OK;
@@ -307,7 +308,7 @@ char **argv;			/* Argument strings. */
 
 
 
-    queue_packet_for_sending(tx_fd, cur_line, cur_line->lineid, &sendq, LINEMSG);
+    queue_packet_for_sending(cur_line, cur_line->lineid, &sendq, LINEMSG);
     /* send a deleted line transmission.. */
     add_to_recent_list(cur_line->lineid, &t, LINEMSG);	/* & add to recent list */
 
@@ -348,14 +349,14 @@ char **argv;			/* Argument strings. */
     b = find_block_by_id(p, argv[1]);
 
     if (b != NULL) {
-	if (b->status == NORMAL) {
+	if ((b->status == NORMAL) || (b->status == LOCKED)) {
 	    b->status = DELETED;
 	    get_cur_time(&t);
 	    time_copy(&t, &(b->last_mod));
 	    time_copy(&t, &(p->last_mod));
 
 	    add_to_recent_list(b->blockid, &t, BLOCKMSG);
-	    queue_packet_for_sending(tx_fd, b, b->blockid,
+	    queue_packet_for_sending(b, b->blockid,
 				     &sendq, BLOCKMSG);
 
 	    /* set the last deleted block as this block - for undelete function */
@@ -443,7 +444,7 @@ char **argv;			/* Argument strings. */
 	time_copy(&t, &(p->last_mod));
 
 	add_to_recent_list(c->blockid, &t, BLOCKMSG);
-	queue_packet_for_sending(tx_fd, c, c->blockid,
+	queue_packet_for_sending(c, c->blockid,
 				 &sendq, BLOCKMSG);
 
 
@@ -467,7 +468,7 @@ char **argv;			/* Argument strings. */
 	/* we'll put everything on our recent list anyway.. */
 	debug("block transmission.\n");
 
-	queue_packet_for_sending(tx_fd, b, b->blockid, &sendq, BLOCKMSG);
+	queue_packet_for_sending(b, b->blockid, &sendq, BLOCKMSG);
 	add_to_recent_list(b->blockid, &t, BLOCKMSG);
 
 	/* now have to copy line data to new lines, so that we can retransmit them as new lines.. */
@@ -515,7 +516,7 @@ char **argv;			/* Argument strings. */
 		debug("Line transmitting \n");
 
 		/* transmit line L */
-		queue_packet_for_sending(tx_fd, l, l->lineid, &sendq, LINEMSG);
+		queue_packet_for_sending(l, l->lineid, &sendq, LINEMSG);
 		add_to_recent_list(l->lineid, &t, LINEMSG);
 
 		y = l;		/* keep 'back' pointer */
@@ -541,7 +542,7 @@ char **argv;			/* Argument strings. */
 	/* will this need extending for each line in block? */
 
 	renumber_block(b);	/* renumber all of the lines in block b. */
-
+	b->new_data=1;
 	redisplay_block(b);	/* re display the block */
 
 
@@ -773,7 +774,7 @@ int ui_paste_block()
 	/* we'll put everything on our recent list anyway.. */
 	debug("block transmission.\n");
 
-	queue_packet_for_sending(tx_fd, b, b->blockid, &sendq, BLOCKMSG);
+	queue_packet_for_sending(b, b->blockid, &sendq, BLOCKMSG);
 	add_to_recent_list(b->blockid, &t, BLOCKMSG);
 
 	/* now have to copy line data to new lines, so that we can retransmit them as new lines.. */
@@ -816,7 +817,7 @@ int ui_paste_block()
 	    debug("Line transmitting \n");
 
 	    /* transmit line L */
-	    queue_packet_for_sending(tx_fd, l, l->lineid, &sendq, LINEMSG);
+	    queue_packet_for_sending(l, l->lineid, &sendq, LINEMSG);
 	    add_to_recent_list(l->lineid, &t, LINEMSG);
 
 	    y = l;		/* keep 'back' pointer */
@@ -841,7 +842,7 @@ int ui_paste_block()
 	/* will this need extending for each line in block? */
 
 	renumber_block(b);	/* renumber all of the lines in block b. */
-
+	b->new_data=1;
 	redisplay_block(b);	/* re display the block */
 
 
@@ -997,7 +998,7 @@ char **argv;			/* Argument strings. */
 
 
 
-	queue_packet_for_sending(tx_fd, new_ln, new_ln->lineid, &sendq, LINEMSG);
+	queue_packet_for_sending(new_ln, new_ln->lineid, &sendq, LINEMSG);
 	add_to_recent_list(new_ln->lineid, &t, LINEMSG);
 	l = new_ln;
 	y++;
@@ -1006,6 +1007,7 @@ char **argv;			/* Argument strings. */
     }
 
     renumber_block(cur_block);	/*renumber the block correctly */
+	cur_block->new_data=1;
     redisplay_block(cur_block);
     return TCL_OK;
 }
@@ -1151,7 +1153,7 @@ char **argv;			/* Argument strings. */
 	get_cur_time(&t);
 	time_copy(&t, &(cur_block->last_mod));
 	time_copy(&t, &(p->last_mod));
-	queue_packet_for_sending(tx_fd, cur_block, cur_block->blockid,
+	queue_packet_for_sending(cur_block, cur_block->blockid,
 				 &sendq, BLOCKMSG);
 	add_to_recent_list(cur_block->blockid, &t, BLOCKMSG);
     }
@@ -1174,7 +1176,7 @@ char **argv;			/* Argument strings. */
 	get_cur_time(&t);
 	time_copy(&t, &(cur_block->last_mod));
 	time_copy(&t, &(p->last_mod));
-	queue_packet_for_sending(tx_fd, cur_block, cur_block->blockid,
+	queue_packet_for_sending(cur_block, cur_block->blockid,
 				 &sendq, BLOCKMSG);
 	add_to_recent_list(cur_block->blockid, &t, BLOCKMSG);
     }
@@ -1198,7 +1200,7 @@ char **argv;			/* Argument strings. */
 	get_cur_time(&t);
 	time_copy(&t, &(cur_block->last_mod));
 	time_copy(&t, &(p->last_mod));
-	queue_packet_for_sending(tx_fd, cur_block, cur_block->blockid,
+	queue_packet_for_sending(cur_block, cur_block->blockid,
 				 &sendq, BLOCKMSG);
 	add_to_recent_list(cur_block->blockid, &t, BLOCKMSG);
     }
@@ -1240,15 +1242,15 @@ int argc;			/* Number of arguments. */
 char **argv;			/* Argument strings. */
 {
     block *b;
-    struct in_addr addr;
     UNUSED(dummy);
     UNUSED(argc);
     b = find_block_by_id(p, argv[1]);
-    if (b == NULL)
-	return TCL_OK;
-    addr.s_addr = htonl(b->originator.hostaddr.s_addr);
+	/* make sure the block is up to date */
+    if (b == NULL) return TCL_OK;
+	redisplay_block (b);
+
     sprintf(interp->result, "%s@%s", b->originator.username,
-	    inet_ntoa(addr));
+	    b->originator.hostaddr);
     return TCL_OK;
 }
 
@@ -1259,15 +1261,13 @@ int argc;			/* Number of arguments. */
 char **argv;			/* Argument strings. */
 {
     block *b;
-    struct in_addr addr;
     UNUSED(dummy);
     UNUSED(argc);
     b = find_block_by_id(p, argv[1]);
-    if (b == NULL)
-	return TCL_OK;
-    addr.s_addr = htonl(b->modifier.hostaddr.s_addr);
+    if (b == NULL) return TCL_OK;
+
     sprintf(interp->result, "%s@%s", b->modifier.username,
-	    inet_ntoa(addr));
+	    b->originator.hostaddr);
     return TCL_OK;
 }
 
@@ -1404,7 +1404,7 @@ char **argv;			/* Argument strings. */
 
     strncpy(p.username, me.username, MAX_USER_NAME);
     strncpy(p.style, argv[4], STYLE_LEN);
-    queue_packet_for_sending(tx_fd, &p, p.id,
+    queue_packet_for_sending(&p, p.id,
 			     &sendq, POINTERMSG);
     return TCL_OK;
 }
@@ -1428,16 +1428,18 @@ char **argv;			/* Argument strings. */
     UNUSED(argc);
     pt = get_participant_by_number(atoi(argv[1]), &plist);
     /* jim fix - replace final variable substitution %u by %lu */
-    sprintf(interp->result, "{%s} {%s} {%s} %d %lu", pt->text_name,
-	    pt->nt_version, pt->os_version, (int) pt->col, pt->time);
-    return TCL_OK;
+	if (pt != NULL) {
+		sprintf(interp->result, "{%s} {%s} {%s} %d %lu %i", pt->text_name,
+		 pt->nt_version, pt->os_version, (int) pt->col, pt->time, pt->packets);
+	}
+	return TCL_OK;
 }
 
 int uigetustimeofday()
 {
     struct timeval tp;
     gettimeofday(&tp, NULL);
-    sprintf(interp->result, "%lu.%lu", tp.tv_sec, tp.tv_usec);
+    sprintf(interp->result, "%ld.%ld", (unsigned long) tp.tv_sec, (unsigned long) tp.tv_usec);
     return TCL_OK;
 }
 
@@ -1494,3 +1496,26 @@ char **argv;
     return TCL_OK;
 
 }
+
+int ui_active_block(dummy, interp, argc, argv)
+ClientData dummy;		/* Not used. */
+Tcl_Interp *interp;		/* Current interpreter. */
+int argc;			/* Number of arguments. */
+char **argv;
+{
+	block *b;
+    UNUSED(dummy);
+    UNUSED(interp);
+    UNUSED(argc);
+
+    if (strcmp (argv[1],"1")==0) {
+	    minimise_updates = 1;	
+    } else if (strcmp (argv[1],"0")==0) {
+	    minimise_updates = 0;	
+	} else {
+        b = find_block_by_id(p, argv[1]);
+		active_block = b;
+	}
+
+    return TCL_OK;
+}
